@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2013-2018 Matt Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@ GNU General Public License for more details.
 #include "ui_importexportdialog.h"
 #include <QFileInfo>
 #include <QDialogButtonBox>
+#include "filedialog.h"
 
 ImportExportDialog::ImportExportDialog(QWidget* parent, Mode eMode, FileType eFileType) : QDialog(parent)
 {
@@ -28,7 +29,6 @@ ImportExportDialog::ImportExportDialog(QWidget* parent, Mode eMode, FileType eFi
     ui = new Ui::ImportExportDialog;
     ui->setupUi(this);
     hidePreviewGroupBox(true);
-    m_fileDialog = new FileDialog(this);
     connect(ui->browseButton, &QPushButton::clicked, this, &ImportExportDialog::browse);
 
     Qt::WindowFlags eFlags = Qt::Dialog | Qt::WindowTitleHint | Qt::WindowCloseButtonHint;
@@ -77,10 +77,10 @@ void ImportExportDialog::init()
     switch (mMode)
     {
         case Import:
-            m_filePaths = QStringList(m_fileDialog->getLastOpenPath(mFileType));
+            m_filePaths = QStringList(FileDialog::getLastOpenPath(mFileType));
             break;
         case Export:
-            m_filePaths = QStringList(m_fileDialog->getLastSavePath(mFileType));
+            m_filePaths = QStringList(FileDialog::getLastSavePath(mFileType));
             break;
         default:
             Q_ASSERT(false);
@@ -110,7 +110,7 @@ QGroupBox* ImportExportDialog::getPreviewGroupBox()
     return ui->previewGroupBox;
 }
 
-void ImportExportDialog::setFileExtension(QString extension)
+void ImportExportDialog::setFileExtension(const QString& extension)
 {
     for (int i = 0; i < m_filePaths.size(); i++)
     {
@@ -130,13 +130,13 @@ void ImportExportDialog::browse()
         case Import:
             if (mFileType == FileType::IMAGE_SEQUENCE)
             {
-                filePaths = m_fileDialog->openFiles( FileType::IMAGE_SEQUENCE );
+                filePaths = FileDialog::getOpenFileNames(this, FileType::IMAGE_SEQUENCE);
                 break;
             }
-            filePaths = QStringList(m_fileDialog->openFile(mFileType));
+            filePaths = QStringList(FileDialog::getOpenFileName(this, mFileType));
             break;
         case Export:
-            filePaths = QStringList(m_fileDialog->saveFile(mFileType));
+            filePaths = QStringList(FileDialog::getSaveFileName(this, mFileType));
             break;
         default:
             Q_ASSERT(false);
