@@ -1,9 +1,9 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
 Copyright (C) 2008-2009 Mj Mendoza IV
-Copyright (C) 2011-2015 Matt Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,9 +19,6 @@ GNU General Public License for more details.
 #define MAINWINDOW2_H
 
 #include <QMainWindow>
-#include "preferencemanager.h"
-#include "pegbaralignmentdialog.h"
-
 
 template<typename T> class QList;
 class QActionGroup;
@@ -44,7 +41,9 @@ class Timeline2;
 class ActionCommands;
 class ImportImageSeqDialog;
 class BackupElement;
-
+class QLabel;
+class PegBarAlignmentDialog;
+enum class SETTING;
 
 
 namespace Ui
@@ -68,66 +67,60 @@ public slots:
     void updateSaveState();
     void clearRecentFilesList();
     void openPegAlignDialog();
-    void closePegAlignDialog();
     void currentLayerChanged();
     void selectionChanged();
+    void viewFlipped();
 
 public:
-    void newDocument(bool force = false);
+    void newDocument();
     void openDocument();
     bool saveDocument();
     bool saveAsNewDocument();
     bool maybeSave();
     bool autoSave();
+    void emptyDocumentWhenErrorOccurred();
 
     // import
     void importImage();
     void importImageSequence();
-    void importImageSequenceNumbered();
-    void addLayerByFilename(QString strFilePath);
     void importPredefinedImageSet();
     void importLayers();
     void importMovieVideo();
     void importGIF();
-    void importMovieAudio();
 
     void lockWidgets(bool shouldLock);
 
-    void setSoundScrubActive(bool b);
-    void setSoundScrubMsec(int msec);
     void setOpacity(int opacity);
     void preferences();
- 
-    void openFile(QString filename);
+
+    void openFile(const QString& filename);
 
     PreferencesDialog* getPrefDialog() { return mPrefDialog; }
 
     void displayMessageBox(const QString& title, const QString& body);
     void displayMessageBoxNoTitle(const QString& body);
 
-Q_SIGNALS:
+signals:
     void updateRecentFilesList(bool b);
 
 protected:
     void tabletEvent(QTabletEvent*) override;
     void closeEvent(QCloseEvent*) override;
-
-private slots:
-    void resetAndDockAllSubWidgets();
+    void showEvent(QShowEvent*) override;
 
 private:
-    bool newObject();
+    void newObject();
     bool newObjectFromPresets(int presetIndex);
-    bool openObject(QString strFilename);
+    bool openObject(const QString& strFilename);
     bool saveObject(QString strFileName);
 
     void createDockWidgets();
     void createMenus();
-    void setMenuActionChecked(QAction*, bool bChecked);
     void setupKeyboardShortcuts();
     void clearKeyboardShortcuts();
     void updateZoomLabel();
-    void tryLoadPreset();
+    bool loadMostRecent();
+    bool tryLoadPreset();
 
     void openPalette();
     void importPalette();
@@ -135,6 +128,7 @@ private:
 
     void readSettings();
     void writeSettings();
+    void resetAndDockAllSubWidgets();
 
     void changePlayState(bool isPlaying);
 
@@ -148,7 +142,8 @@ private:
     void makeConnections(Editor*, ToolOptionWidget*);
     void makeConnections(Editor*, OnionSkinWidget*);
 
-    void bindActionWithSetting(QAction*, SETTING);
+    bool tryRecoverUnsavedProject();
+    void startProjectRecovery(int result);
 
     // UI: Dock widgets
     ColorBox*             mColorBox = nullptr;
@@ -156,11 +151,11 @@ private:
     DisplayOptionWidget*  mDisplayOptionWidget = nullptr;
     ToolOptionWidget*     mToolOptions = nullptr;
     ToolBoxWidget*        mToolBox = nullptr;
-    Timeline2*            mTimeline2 = nullptr;
+    //Timeline2*          mTimeline2 = nullptr;
     RecentFileMenu*       mRecentFileMenu = nullptr;
     PreferencesDialog*    mPrefDialog = nullptr;
     //PreviewWidget*      mPreview = nullptr;
-    TimeLine*             mTimeLine = nullptr; // be public temporary
+    TimeLine*             mTimeLine = nullptr;
     ColorInspector*       mColorInspector = nullptr;
     OnionSkinWidget*      mOnionSkinWidget = nullptr;
 
@@ -169,7 +164,6 @@ private:
 
     PegBarAlignmentDialog* mPegAlign = nullptr;
 
-private:
     ActionCommands* mCommands = nullptr;
     QList<BaseDockWidget*> mDockWidgets;
 
@@ -179,9 +173,12 @@ private:
     // a hack for MacOS because closeEvent fires twice
     bool m2ndCloseEvent = false;
 
-    // whether we are currently importing an image sequence.
-    bool mIsImportingImageSequence = false;
-    
+    // statusbar widgets
+    QLabel* mZoomLabel = nullptr;
+
+    // Whether to suppress the auto save dialog due to internal work
+    bool mSuppressAutoSaveDialog = false;
+
     Ui::MainWindow2* ui = nullptr;
 };
 
