@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,7 +20,6 @@ GNU General Public License for more details.
 
 #include <memory>
 #include <QObject>
-#include <QList>
 #include "pencilerror.h"
 #include "pencildef.h"
 
@@ -48,7 +47,6 @@ class ActiveFramePool;
 
 enum class SETTING;
 
-
 class Editor : public QObject
 {
     Q_OBJECT
@@ -64,7 +62,7 @@ class Editor : public QObject
 
 public:
     explicit Editor(QObject* parent = nullptr);
-    virtual ~Editor();
+    ~Editor() override;
 
     bool init();
 
@@ -92,32 +90,40 @@ public:
     int fps();
     void setFps(int fps);
 
-    int  currentLayerIndex() { return mCurrentLayerIndex; }
+    int  currentLayerIndex() const { return mCurrentLayerIndex; }
     void setCurrentLayerIndex(int i);
 
     void scrubTo(int frameNumber);
 
-
     /**
-     * @brief The visiblity value should match any of the VISIBILITY enum values
+     * @brief The visibility value should match any of the VISIBILITY enum values
      */
     void setLayerVisibility(LayerVisibility visibility);
     LayerVisibility layerVisibility();
-    bool exportSeqCLI(QString filePath, LayerCamera* cameraLayer, QString format = "PNG", int width = -1, int height = -1, int startFrame = 1, int endFrame = -1, bool transparency = false, bool antialias = true);
-    bool exportMovieCLI(QString filePath, LayerCamera* cameraLayer, int width = -1, int height = -1, int startFrame = 1, int endFrame = -1);
 
     qreal viewScaleInversed();
     void deselectAll();
     void selectAll();
 
-    QString workingDir() const;
-
     // backup
     int mBackupIndex;
     BackupElement* currentBackup();
     QList<BackupElement*> mBackupList;
 
-Q_SIGNALS:
+signals:
+
+    /** This should be emitted after scrubbing */
+    void scrubbed(int frameNumber);
+
+    /** This should be emitted after modifying the frame content */
+    void frameModified(int frameNumber);
+
+    /** This should be emitted after the object has been changed */
+    void objectChanged();
+
+    /** This should be emitted after modifying multiple frames */
+    void framesModified();
+
     void updateTimeLine();
     void updateLayerCount();
     void updateBackup();
@@ -125,25 +131,33 @@ Q_SIGNALS:
     void objectLoaded();
 
     void changeThinLinesButton(bool);
-    void currentFrameChanged(int n);
+    void fpsChanged(int fps);
 
     void needSave();
     void needDisplayInfo(const QString& title, const QString& body);
     void needDisplayInfoNoTitle(const QString& body);
 
 public: //slots
+
+    /** Will call update() and update the canvas
+     * Only call this directly If you need the cache to be intact and require the frame to be repainted
+     * Convenient method that does the same as updateFrame but for the current frame
+    */
+    void updateCurrentFrame();
+
+    /** Will call update() and update the canvas
+     * Only call this directly If you need the cache to be intact and require the frame to be repainted
+    */
+    void updateFrame(int frameNumber);
+
     void clearCurrentFrame();
 
     void cut();
 
     bool importImage(QString filePath);
     bool importGIF(QString filePath, int numOfImages = 0);
-    void updateFrame(int frameNumber);
     void restoreKey();
 
-    void updateFrameAndVector(int frameNumber);
-    void updateCurrentFrame();
-
     void scrubNextKeyFrame();
     void scrubPreviousKeyFrame();
     void scrubForward();
@@ -154,12 +168,22 @@ public: //slots
 
     void notifyAnimationLengthChanged();
     void switchVisibilityOfLayer(int layerNumber);
-    void showLayerNotVisibleWarning();
     void swapLayers(int i, int j);
-    Status::StatusInt pegBarAlignment(QStringList layers);
+    Status pegBarAlignment(QStringList layers);
 
     void backup(QString undoText);
     void backup(int layerNumber, int frameNumber, QString undoText);
+    /**
+     * Restores integrity of the backup elements after a layer has been deleted.
+     * Removes backup elements affecting the deleted layer and adjusts the layer
+     * index on other backup elements as necessary.
+     *
+     * @param layerIndex The index of the layer that was deleted
+     *
+     * @warning This serves as a temporary hack to prevent crashes until #864 is done
+     *          (see #1412).
+     */
+    void sanitizeBackupElementsAfterLayerDeletion(int layerIndex);
     void undo();
     void redo();
     void copy();
@@ -170,15 +194,15 @@ public: //slots
     void decreaseLayerVisibilityIndex();
     void flipSelection(bool flipVertical);
 
-    void toogleOnionSkinType();
+    void toggleOnionSkinType();
 
     void clearTemporary();
-    void addTemporaryDir(QTemporaryDir* const dir);
+    void addTemporaryDir(QTemporaryDir* dir);
 
     void settingUpdated(SETTING);
 
     void dontAskAutoSave(bool b) { mAutosaveNeverAskAgain = b; }
-    bool autoSaveNeverAskAgain() { return mAutosaveNeverAskAgain; }
+    bool autoSaveNeverAskAgain() const { return mAutosaveNeverAskAgain; }
     void resetAutoSaveCounter();
 
     void createNewBitmapLayer(const QString& name);
@@ -196,7 +220,7 @@ private:
     bool importVectorImage(QString);
 
     // the object to be edited by the editor
-    std::shared_ptr<Object> mObject = nullptr;
+    std::unique_ptr<Object> mObject;
 
     int mFrame = 1; // current frame number.
     int mCurrentLayerIndex = 0; // the current layer to be edited/displayed
@@ -214,8 +238,6 @@ private:
 
     std::vector< BaseManager* > mAllManagers;
 
-    bool mIsAltPressed = false;
-
     bool mIsAutosave = true;
     int mAutosaveNumber = 12;
     int mAutosaveCounter = 0;
