@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -39,10 +39,11 @@ bool LayerManager::init()
     return true;
 }
 
-Status LayerManager::load(Object* o)
+Status LayerManager::load(Object*)
 {
     mLastCameraLayerIdx = 0;
-    emit layerCountChanged(o->getLayerCount());
+    // Do not emit layerCountChanged here because the editor has not updated to this object yet
+    // Leave that to the caller of this function
     return Status::OK;
 }
 
@@ -71,7 +72,9 @@ Layer* LayerManager::getLastCameraLayer()
 
 Layer* LayerManager::currentLayer()
 {
-    return currentLayer(0);
+    Layer* layer = currentLayer(0);
+    Q_ASSERT(layer != nullptr);
+    return layer;
 }
 
 Layer* LayerManager::currentLayer(int incr)
@@ -110,7 +113,7 @@ void LayerManager::setCurrentLayer(int layerIndex)
     // Do not check if layer index has changed
     // because the current layer may have changed either way
     editor()->setCurrentLayerIndex(layerIndex);
-    Q_EMIT currentLayerChanged(layerIndex);
+    emit currentLayerChanged(layerIndex);
 
     if (object())
     {
@@ -131,7 +134,7 @@ void LayerManager::gotoNextLayer()
     if (editor()->currentLayerIndex() < object()->getLayerCount() - 1)
     {
         editor()->setCurrentLayerIndex(editor()->currentLayerIndex() + 1);
-        Q_EMIT currentLayerChanged(editor()->currentLayerIndex());
+        emit currentLayerChanged(editor()->currentLayerIndex());
     }
 }
 
@@ -140,16 +143,44 @@ void LayerManager::gotoPreviouslayer()
     if (editor()->currentLayerIndex() > 0)
     {
         editor()->setCurrentLayerIndex(editor()->currentLayerIndex() - 1);
-        Q_EMIT currentLayerChanged(editor()->currentLayerIndex());
+        emit currentLayerChanged(editor()->currentLayerIndex());
     }
 }
 
+QString LayerManager::nameSuggestLayer(const QString& name)
+{
+    // if no layers: return name
+    if (count() == 0)
+    {
+        return name;
+    }
+    QVector<QString> sLayers;
+    // fill Vector with layer names
+    for (int i = 0; i < count(); i++)
+    {
+        sLayers.append(getLayer(i)->name());
+    }
+    // if name is not in list, return name
+    if (!sLayers.contains(name))
+    {
+        return name;
+    }
+    int newIndex = 2;
+    QString newName = name;
+    do {
+        newName = QStringLiteral("%1 %2")
+            .arg(name).arg(QString::number(newIndex++));
+    } while (sLayers.contains(newName));
+    return newName;
+}
+
 LayerBitmap* LayerManager::createBitmapLayer(const QString& strLayerName)
 {
     LayerBitmap* layer = object()->addNewBitmapLayer();
     layer->setName(strLayerName);
 
-    Q_EMIT layerCountChanged(count());
+    emit layerCountChanged(count());
+    setCurrentLayer(getLastLayerIndex());
 
     return layer;
 }
@@ -159,7 +190,8 @@ LayerVector* LayerManager::createVectorLayer(const QString& strLayerName)
     LayerVector* layer = object()->addNewVectorLayer();
     layer->setName(strLayerName);
 
-    Q_EMIT layerCountChanged(count());
+    emit layerCountChanged(count());
+    setCurrentLayer(getLastLayerIndex());
 
     return layer;
 }
@@ -169,7 +201,8 @@ LayerCamera* LayerManager::createCameraLayer(const QString& strLayerName)
     LayerCamera* layer = object()->addNewCameraLayer();
     layer->setName(strLayerName);
 
-    Q_EMIT layerCountChanged(count());
+    emit layerCountChanged(count());
+    setCurrentLayer(getLastLayerIndex());
 
     return layer;
 }
@@ -179,12 +212,13 @@ LayerSound* LayerManager::createSoundLayer(const QString& strLayerName)
     LayerSound* layer = object()->addNewSoundLayer();
     layer->setName(strLayerName);
 
-    Q_EMIT layerCountChanged(count());
+    emit layerCountChanged(count());
+    setCurrentLayer(getLastLayerIndex());
 
     return layer;
 }
 
-int LayerManager::LastFrameAtFrame(int frameIndex)
+int LayerManager::lastFrameAtFrame(int frameIndex)
 {
     Object* o = object();
     for (int i = frameIndex; i >= 0; i -= 1)
@@ -265,8 +299,8 @@ Status LayerManager::deleteLayer(int index)
         setCurrentLayer(currentLayerIndex());
     }
 
-    Q_EMIT layerDeleted(index);
-    Q_EMIT layerCountChanged(count());
+    emit layerDeleted(index);
+    emit layerCountChanged(count());
 
     return Status::OK;
 }
@@ -325,7 +359,7 @@ int LayerManager::animationLength(bool includeSounds)
 
 void LayerManager::notifyAnimationLengthChanged()
 {
-    emit animationLengthChanged(animationLength(false));
+    emit animationLengthChanged(animationLength(true));
 }
 
 int LayerManager::getIndex(Layer* layer) const
