@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -18,7 +18,7 @@ GNU General Public License for more details.
 #include "preferencemanager.h"
 
 #include <QSettings>
-
+#include <QDebug>
 
 PreferenceManager::PreferenceManager(Editor* editor) : BaseManager(editor)
 {
@@ -82,16 +82,19 @@ void PreferenceManager::loadPrefs()
     set(SETTING::BACKGROUND_STYLE,         settings.value(SETTING_BACKGROUND_STYLE,       "white").toString());
 
     set(SETTING::LAYOUT_LOCK,              settings.value(SETTING_LAYOUT_LOCK,            false).toBool());
-    set(SETTING::FRAME_POOL_SIZE,          settings.value(SETTING_FRAME_POOL_SIZE,        200).toInt());
+    set(SETTING::FRAME_POOL_SIZE,          settings.value(SETTING_FRAME_POOL_SIZE,        1024).toInt());
 
     set(SETTING::FPS,                      settings.value(SETTING_FPS,                    12).toInt());
     set(SETTING::FIELD_W,                  settings.value(SETTING_FIELD_W,                800).toInt());
     set(SETTING::FIELD_H,                  settings.value(SETTING_FIELD_H,                600).toInt());
+    set(SETTING::TIMECODE_TEXT,            settings.value(SETTING_TIMECODE_TEXT,          1).toInt());
 
     // Files
     set(SETTING::AUTO_SAVE,                settings.value(SETTING_AUTO_SAVE,              false).toBool());
     set(SETTING::AUTO_SAVE_NUMBER,         settings.value(SETTING_AUTO_SAVE_NUMBER,       256).toInt());
     set(SETTING::ASK_FOR_PRESET,           settings.value(SETTING_ASK_FOR_PRESET,         false).toBool());
+    set(SETTING::LOAD_MOST_RECENT,         settings.value(SETTING_LOAD_MOST_RECENT,       false).toBool());
+    set(SETTING::LOAD_DEFAULT_PRESET,      settings.value(SETTING_LOAD_DEFAULT_PRESET,    true).toBool());
     set(SETTING::DEFAULT_PRESET,           settings.value(SETTING_DEFAULT_PRESET,         0).toInt());
 
     // Timeline
@@ -281,6 +284,9 @@ void PreferenceManager::set(SETTING option, int value)
     case SETTING::GRID_SIZE_W:
         settings.setValue(SETTING_GRID_SIZE_W, value);
         break;
+    case SETTING::TIMECODE_TEXT:
+        settings.setValue(SETTING_TIMECODE_TEXT, value);
+        break;
     case SETTING::GRID_SIZE_H:
         settings.setValue(SETTING_GRID_SIZE_H, value);
         break;
@@ -418,6 +424,12 @@ void PreferenceManager::set(SETTING option, bool value)
     case SETTING::ASK_FOR_PRESET:
         settings.setValue(SETTING_ASK_FOR_PRESET, value);
         break;
+    case SETTING::LOAD_MOST_RECENT:
+        settings.setValue(SETTING_LOAD_MOST_RECENT, value);
+        break;
+    case SETTING::LOAD_DEFAULT_PRESET:
+        settings.setValue(SETTING_LOAD_DEFAULT_PRESET, value);
+        break;
     default:
         Q_ASSERT(false);
         break;
