@@ -1,7 +1,7 @@
 /*
 
-Pencil - Traditional Animation Software
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Pencil2D - Traditional Animation Software
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -177,7 +177,10 @@ Status MovieExporter::assembleAudio(const Object* obj,
     {
         layer->foreachKeyFrame([&allSoundClips](KeyFrame* key)
         {
-            allSoundClips.push_back(static_cast<SoundClip*>(key));
+            if (!key->fileName().isEmpty())
+            {
+                allSoundClips.push_back(static_cast<SoundClip*>(key));
+            }
         });
     }
 
