@@ -1,7 +1,7 @@
 /*
 
-Pencil - Traditional Animation Software
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Pencil2D - Traditional Animation Software
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -16,58 +16,53 @@ GNU General Public License for more details.
 
 #include "soundplayer.h"
 #include <QMediaPlayer>
-#include <QBuffer>
 #include <QFile>
 #include "soundclip.h"
 
-SoundPlayer::SoundPlayer( )
+SoundPlayer::SoundPlayer()
 {
-
 }
 
 SoundPlayer::~SoundPlayer()
 {
 }
 
-void SoundPlayer::init( SoundClip* clip )
+void SoundPlayer::init(SoundClip* clip)
 {
-    Q_ASSERT( clip != nullptr );
+    Q_ASSERT(clip != nullptr);
     mSoundClip = clip;
 
-    mMediaPlayer = new QMediaPlayer( this );
+    mMediaPlayer = new QMediaPlayer(this);
 
     QFile file(clip->fileName());
     file.open(QIODevice::ReadOnly);
-    QBuffer *buffer = new QBuffer();
-    buffer->setData(file.readAll());
-    buffer->open(QBuffer::ReadOnly);
 
-    mMediaPlayer->setMedia( QUrl::fromLocalFile( clip->fileName() ), buffer );
+    mBuffer.setData(file.readAll());
+    mBuffer.open(QBuffer::ReadOnly);
+
+    mMediaPlayer->setMedia(QUrl::fromLocalFile(clip->fileName()), &mBuffer);
     makeConnections();
 
-    clip->attachPlayer( this );
-    //mMediaPlayer->play();
-
-    qDebug() << "Seekable = " << mMediaPlayer->isSeekable();
+    clip->attachPlayer(this);
 }
 
-void SoundPlayer::onKeyFrameDestroy( KeyFrame* keyFrame )
+void SoundPlayer::onKeyFrameDestroy(KeyFrame* keyFrame)
 {
     Q_UNUSED(keyFrame)
 }
 
 bool SoundPlayer::isValid()
 {
-    if ( mMediaPlayer )
+    if (mMediaPlayer)
     {
-        return ( mMediaPlayer->error() == QMediaPlayer::NoError );
+        return (mMediaPlayer->error() == QMediaPlayer::NoError);
     }
     return false;
 }
 
 void SoundPlayer::play()
 {
-    if ( mMediaPlayer )
+    if (mMediaPlayer)
     {
         mMediaPlayer->play();
     }
@@ -75,7 +70,7 @@ void SoundPlayer::play()
 
 void SoundPlayer::pause()
 {
-    if ( mMediaPlayer )
+    if (mMediaPlayer)
     {
         mMediaPlayer->pause();
     }
@@ -83,7 +78,7 @@ void SoundPlayer::pause()
 
 void SoundPlayer::stop()
 {
-    if ( mMediaPlayer )
+    if (mMediaPlayer)
     {
         mMediaPlayer->stop();
     }
@@ -91,7 +86,7 @@ void SoundPlayer::stop()
 
 int64_t SoundPlayer::duration()
 {
-    if ( mMediaPlayer )
+    if (mMediaPlayer)
     {
         return mMediaPlayer->duration();
     }
@@ -100,23 +95,23 @@ int64_t SoundPlayer::duration()
 
 void SoundPlayer::setMediaPlayerPosition(qint64 pos)
 {
-    if( mMediaPlayer )
+    if (mMediaPlayer)
     {
         mMediaPlayer->setPosition(pos);
     }
 }
 
 void SoundPlayer::makeConnections()
-{   
-    auto errorSignal = static_cast< void ( QMediaPlayer::* )( QMediaPlayer::Error ) >( &QMediaPlayer::error );
-    connect( mMediaPlayer, errorSignal, this, []( QMediaPlayer::Error err )
+{
+    auto errorSignal = static_cast<void (QMediaPlayer::*)(QMediaPlayer::Error)>(&QMediaPlayer::error);
+    connect(mMediaPlayer, errorSignal, this, [](QMediaPlayer::Error err)
     {
         qDebug() << "MediaPlayer Error: " << err;
-    } );
+    });
 
-    connect( mMediaPlayer, &QMediaPlayer::durationChanged, [ this ]( qint64 duration ) 
+    connect(mMediaPlayer, &QMediaPlayer::durationChanged, [this](qint64 duration)
     {
         qDebug() << "MediaPlayer durationChanged :" << duration;
-        emit durationChanged( this, duration );
-    } );
+        emit durationChanged(this, duration);
+    });
 }
