@@ -1,7 +1,7 @@
 /*
 
-Pencil - Traditional Animation Software
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Pencil2D - Traditional Animation Software
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@ GNU General Public License for more details.
 
 #include <memory>
 #include <QObject>
+#include <QBuffer>
 #include "pencilerror.h"
 #include "keyframe.h"
 
@@ -32,28 +33,29 @@ public:
     SoundPlayer();
     ~SoundPlayer() override;
 
-    void init( SoundClip* );
-    void onKeyFrameDestroy( KeyFrame* ) override;
+    void init(SoundClip*);
+    void onKeyFrameDestroy(KeyFrame*) override;
     bool isValid();
 
     void play();
     void pause();
     void stop();
-    
+
     int64_t duration();
     SoundClip* clip() { return mSoundClip; }
 
-    void setMediaPlayerPosition( qint64 pos );
+    void setMediaPlayerPosition(qint64 pos);
 
-Q_SIGNALS:
-    void corruptedSoundFile( SoundClip* );
-    void durationChanged( SoundPlayer*, int64_t duration );
+signals:
+    void corruptedSoundFile(SoundClip*);
+    void durationChanged(SoundPlayer*, int64_t duration);
 
 private:
     void makeConnections();
 
     SoundClip* mSoundClip = nullptr;
     QMediaPlayer* mMediaPlayer = nullptr;
+    QBuffer mBuffer;
 };
 
 #endif // SOUNDPLAYER_H
