@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@ GNU General Public License for more details.
 
 #include <ctime>
 #include <QDir>
+#include <QVersionNumber>
 #include "pencildef.h"
 #include "qminiz.h"
 #include "fileformat.h"
@@ -45,7 +46,7 @@ FileManager::FileManager(QObject* parent) : QObject(parent)
     srand(static_cast<uint>(time(nullptr)));
 }
 
-Object* FileManager::load(QString sFileName)
+Object* FileManager::load(const QString& sFileName)
 {
     DebugDetails dd;
     dd << QString("File name: ").append(sFileName);
@@ -59,7 +60,7 @@ Object* FileManager::load(QString sFileName)
 
     progressForward();
 
-    Object* obj = new Object;
+    std::unique_ptr<Object> obj(new Object);
     obj->setFilePath(sFileName);
     obj->createWorkingDir();
 
@@ -72,14 +73,14 @@ Object* FileManager::load(QString sFileName)
 
     if (oldFormat)
     {
-        dd << "Recognized Old Pencil File Format (*.pcl) !";
+        dd << "Recognized Old Pencil2D File Format (*.pcl) !";
 
         strMainXMLFile = sFileName;
         strDataFolder = strMainXMLFile + "." + PFF_OLD_DATA_DIR;
     }
     else
     {
-        dd << "Recognized New zipped Pencil File Format (*.pclx) !";
+        dd << "Recognized New zipped Pencil2D File Format (*.pclx) !";
 
         unzip(sFileName, obj->workingDir());
 
@@ -134,29 +135,29 @@ Object* FileManager::load(QString sFileName)
         return cleanUpWithErrorCode(Status(Status::ERROR_INVALID_PENCIL_FILE, dd, openErrorTitle, openErrorDesc + contactLinks));
     }
 
-    loadPalette(obj);
+    loadPalette(obj.get());
 
     bool ok = true;
 
     if (root.tagName() == "document")
     {
-        ok = loadObject(obj, root);
+        ok = loadObject(obj.get(), root);
     }
     else if (root.tagName() == "object" || root.tagName() == "MyOject") // old Pencil format (<=0.4.3)
     {
-        ok = loadObjectOldWay(obj, root);
+        ok = loadObjectOldWay(obj.get(), root);
     }
 
     if (!ok)
     {
-        delete obj;
+        obj.reset();
         dd << "Issue occurred during object loading";
         return cleanUpWithErrorCode(Status(Status::ERROR_INVALID_PENCIL_FILE, dd, ""));
     }
 
-    verifyObject(obj);
+    verifyObject(obj.get());
 
-    return obj;
+    return obj.release();
 }
 
 bool FileManager::loadObject(Object* object, const QDomElement& root)
@@ -185,6 +186,19 @@ bool FileManager::loadObject(Object* object, const QDomElement& root)
             ObjectData* projectData = loadProjectData(element);
             object->setData(projectData);
         }
+        else if (element.tagName() == "version")
+        {
+            QVersionNumber fileVersion = QVersionNumber::fromString(element.text());
+            QVersionNumber appVersion = QVersionNumber::fromString(APP_VERSION);
+
+            if (!fileVersion.isNull())
+            {
+                if (appVersion < fileVersion)
+                {
+                    qWarning() << "You are opening a newer project file in an older version of Pencil2D!";
+                }
+            }
+        }
         else
         {
             Q_ASSERT(false);
@@ -203,19 +217,25 @@ bool FileManager::isOldForamt(const QString& fileName) const
     return !(MiniZ::isZip(fileName));
 }
 
-Status FileManager::save(Object* object, QString sFileName)
+Status FileManager::save(const Object* object, const QString& sFileName)
 {
     DebugDetails dd;
-    dd << "FileManager::save";
+    dd << __FUNCTION__;
     dd << ("sFileName = " + sFileName);
 
     if (object == nullptr)
     {
-        dd << "object parameter is null";
+        dd << "Object parameter is null";
         return Status(Status::INVALID_ARGUMENT, dd);
     }
+    if (sFileName.isEmpty()) {
+        dd << "File name is empty";
+        return Status(Status::INVALID_ARGUMENT, dd,
+                      tr("Invalid Save Path"),
+                      tr("The path is empty."));
+    }
 
-    int totalCount = object->totalKeyFrameCount();
+    const int totalCount = object->totalKeyFrameCount();
     mMaxProgressValue = totalCount + 5;
     emit progressRangeChanged(mMaxProgressValue);
 
@@ -249,17 +269,17 @@ Status FileManager::save(Object* object, QString sFileName)
     QString sMainXMLFile;
     QString sDataFolder;
 
-    bool isOldType = sFileName.endsWith(PFF_OLD_EXTENSION);
+    const bool isOldType = sFileName.endsWith(PFF_OLD_EXTENSION);
     if (isOldType)
     {
-        dd << "Old Pencil File Format (*.pcl) !";
+        dd << "Old Pencil2D File Format (*.pcl) !";
 
         sMainXMLFile = sFileName;
         sDataFolder = sMainXMLFile + "." + PFF_OLD_DATA_DIR;
     }
     else
     {
-        dd << "New zipped Pencil File Format (*.pclx) !";
+        dd << "New zipped Pencil2D File Format (*.pclx) !";
 
         sTempWorkingFolder = object->workingDir();
         Q_ASSERT(QDir(sTempWorkingFolder).exists());
@@ -291,79 +311,17 @@ Status FileManager::save(Object* object, QString sFileName)
                       tr("\"%1\" is a file. Please delete the file and try again.").arg(dataInfo.absoluteFilePath()));
     }
 
-    // save data
-    int numLayers = object->getLayerCount();
-    dd << QString("Total %1 layers").arg(numLayers);
+    QStringList filesToZip; // A files list in the working folder needs to be zipped
+    Status stKeyFrames = writeKeyFrameFiles(object, sDataFolder, filesToZip);
+    dd.collect(stKeyFrames.details());
 
-    for (int i = 0; i < numLayers; ++i)
-    {
-        Layer* layer = object->getLayer(i);
-        layer->presave(sDataFolder);
-    }
+    Status stMainXml = writeMainXml(object, sMainXMLFile, filesToZip);
+    dd.collect(stMainXml.details());
 
-    QStringList zippedFiles;
+    Status stPalette = writePalette(object, sDataFolder, filesToZip);
+    dd.collect(stPalette.details());
 
-    bool saveLayersOK = true;
-    for (int i = 0; i < numLayers; ++i)
-    {
-        Layer* layer = object->getLayer(i);
-
-        dd << QString("Layer[%1] = [id=%2, name=%3, type=%4]").arg(i).arg(layer->id()).arg(layer->name()).arg(layer->type());
-
-        Status st = layer->save(sDataFolder, zippedFiles, [this] { progressForward(); });
-        if (!st.ok())
-        {
-            saveLayersOK = false;
-            dd.collect(st.details());
-            dd << QString("  !! Failed to save Layer[%1] %2").arg(i).arg(layer->name());
-        }
-    }
-    dd << "All Layers saved";
-
-    // save palette
-    QString sPaletteFile = object->savePalette(sDataFolder);
-    if (!sPaletteFile.isEmpty())
-        zippedFiles.append(sPaletteFile);
-    else
-        dd << "Failed to save the palette xml";
-
-    progressForward();
-
-    // -------- save main XML file -----------
-    QFile file(sMainXMLFile);
-    if (!file.open(QFile::WriteOnly | QFile::Text))
-    {
-        return Status(Status::ERROR_FILE_CANNOT_OPEN, dd);
-    }
-
-    QDomDocument xmlDoc("PencilDocument");
-    QDomElement root = xmlDoc.createElement("document");
-    QDomProcessingInstruction encoding = xmlDoc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
-    xmlDoc.appendChild(encoding);
-    xmlDoc.appendChild(root);
-
-    progressForward();
-
-    // save editor information
-    QDomElement projDataXml = saveProjectData(object->data(), xmlDoc);
-    root.appendChild(projDataXml);
-
-    // save object
-    QDomElement objectElement = object->saveXML(xmlDoc);
-    root.appendChild(objectElement);
-
-    dd << "Writing main xml file...";
-
-    const int indentSize = 2;
-
-    QTextStream out(&file);
-    xmlDoc.save(out, indentSize);
-    out.flush();
-    file.close();
-
-    dd << "Done writing main xml file at" << sMainXMLFile;
-
-    zippedFiles.append(sMainXMLFile);
+    const bool saveOk = stKeyFrames.ok() && stMainXml.ok() && stPalette.ok();
 
     progressForward();
 
@@ -373,23 +331,24 @@ Status FileManager::save(Object* object, QString sFileName)
 
         QString sBackupFile = backupPreviousFile(sFileName);
 
-        Status s = MiniZ::compressFolder(sFileName, sTempWorkingFolder, zippedFiles);
-        if (!s.ok())
+        Status stMiniz = MiniZ::compressFolder(sFileName, sTempWorkingFolder, filesToZip);
+        if (!stMiniz.ok())
         {
-            dd.collect(s.details());
+            dd.collect(stMiniz.details());
             return Status(Status::ERROR_MINIZ_FAIL, dd,
                           tr("Miniz Error"),
                           tr("An internal error occurred. Your file may not be saved successfully."));
         }
         dd << "Zip file saved successfully";
+        Q_ASSERT(stMiniz.ok());
 
-        if (s.ok() && saveLayersOK)
+        if (saveOk)
             deleteBackupFile(sBackupFile);
     }
 
     progressForward();
 
-    if (!saveLayersOK)
+    if (!saveOk)
     {
         return Status(Status::FAIL, dd,
                       tr("Internal Error"),
@@ -399,6 +358,29 @@ Status FileManager::save(Object* object, QString sFileName)
     return Status::OK;
 }
 
+Status FileManager::writeToWorkingFolder(const Object* object)
+{
+    DebugDetails dd;
+
+    QStringList filesWritten;
+
+    const QString dataFolder = object->dataDir();
+    const QString mainXml = object->mainXMLFile();
+
+    Status stKeyFrames = writeKeyFrameFiles(object, dataFolder, filesWritten);
+    dd.collect(stKeyFrames.details());
+
+    Status stMainXml = writeMainXml(object, mainXml, filesWritten);
+    dd.collect(stMainXml.details());
+
+    Status stPalette = writePalette(object, dataFolder, filesWritten);
+    dd.collect(stPalette.details());
+
+    const bool saveOk = stKeyFrames.ok() && stMainXml.ok() && stPalette.ok();
+    const auto errorCode = (saveOk) ? Status::OK : Status::FAIL;
+    return Status(errorCode, dd);
+}
+
 ObjectData* FileManager::loadProjectData(const QDomElement& docElem)
 {
     ObjectData* data = new ObjectData;
@@ -589,6 +571,102 @@ bool FileManager::loadPalette(Object* obj)
     return true;
 }
 
+Status FileManager::writeKeyFrameFiles(const Object* object, const QString& dataFolder, QStringList& filesFlushed)
+{
+    DebugDetails dd;
+
+    const int numLayers = object->getLayerCount();
+    dd << QString("Total %1 layers").arg(numLayers);
+
+    for (int i = 0; i < numLayers; ++i)
+    {
+        Layer* layer = object->getLayer(i);
+        layer->presave(dataFolder);
+    }
+
+    bool saveLayersOK = true;
+    for (int i = 0; i < numLayers; ++i)
+    {
+        Layer* layer = object->getLayer(i);
+
+        dd << QString("Layer[%1] = [id=%2, name=%3, type=%4]").arg(i).arg(layer->id()).arg(layer->name()).arg(layer->type());
+
+        Status st = layer->save(dataFolder, filesFlushed, [this] { progressForward(); });
+        if (!st.ok())
+        {
+            saveLayersOK = false;
+            dd.collect(st.details());
+            dd << QString("  !! Failed to save Layer[%1] %2").arg(i).arg(layer->name());
+        }
+    }
+    dd << "All Layers saved";
+
+    progressForward();
+
+    auto errorCode = (saveLayersOK) ? Status::OK : Status::FAIL;
+    return Status(errorCode, dd);
+}
+
+Status FileManager::writeMainXml(const Object* object, const QString& mainXml, QStringList& filesWritten)
+{
+    DebugDetails dd;
+
+    QFile file(mainXml);
+    if (!file.open(QFile::WriteOnly | QFile::Text))
+    {
+        dd << "Failed to open Main XML" << mainXml;
+        return Status(Status::ERROR_FILE_CANNOT_OPEN, dd);
+    }
+
+    QDomDocument xmlDoc("PencilDocument");
+    QDomElement root = xmlDoc.createElement("document");
+    QDomProcessingInstruction encoding = xmlDoc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
+    xmlDoc.appendChild(encoding);
+    xmlDoc.appendChild(root);
+
+    progressForward();
+
+    // save editor information
+    QDomElement projDataXml = saveProjectData(object->data(), xmlDoc);
+    root.appendChild(projDataXml);
+
+    // save object
+    QDomElement objectElement = object->saveXML(xmlDoc);
+    root.appendChild(objectElement);
+
+    // save Pencil2D version
+    QDomElement versionElem = xmlDoc.createElement("version");
+    versionElem.appendChild(xmlDoc.createTextNode(QString(APP_VERSION)));
+    root.appendChild(versionElem);
+
+    dd << "Writing main xml file...";
+
+    const int indentSize = 2;
+
+    QTextStream out(&file);
+    xmlDoc.save(out, indentSize);
+    out.flush();
+    file.close();
+
+    dd << "Done writing main xml file: " << mainXml;
+
+    filesWritten.append(mainXml);
+    return Status(Status::OK, dd);
+}
+
+Status FileManager::writePalette(const Object* object, const QString& dataFolder, QStringList& filesWritten)
+{
+    const QString paletteFile = object->savePalette(dataFolder);
+    if (paletteFile.isEmpty())
+    {
+        DebugDetails dd;
+        dd << "Failed to save palette";
+        return Status(Status::FAIL, dd);
+    }
+    filesWritten.append(paletteFile);
+    return Status::OK;
+}
+
 void FileManager::unzip(const QString& strZipFile, const QString& strUnzipTarget)
 {
     // removes the previous directory first  - better approach
@@ -630,3 +708,248 @@ Status FileManager::verifyObject(Object* obj)
     }
     return Status::OK;
 }
+
+QStringList FileManager::searchForUnsavedProjects()
+{
+    QDir pencil2DTempDir = QDir::temp();
+    bool folderExists = pencil2DTempDir.cd("Pencil2D");
+    if (!folderExists)
+    {
+        return QStringList();
+    }
+
+    const QStringList nameFilter("*_" PFF_TMP_DECOMPRESS_EXT "_*"); // match name pattern like "Default_Y2xD_0a4e44e9"
+    QStringList entries = pencil2DTempDir.entryList(nameFilter, QDir::Dirs | QDir::Readable);
+
+    QStringList recoverables;
+    for (const QString path : entries)
+    {
+        QString fullPath = pencil2DTempDir.filePath(path);
+        if (isProjectRecoverable(fullPath))
+        {
+            qDebug() << "Found debris at" << fullPath;
+            recoverables.append(fullPath);
+        }
+    }
+    return recoverables;
+}
+
+bool FileManager::isProjectRecoverable(const QString& projectFolder)
+{
+    QDir dir(projectFolder);
+    if (!dir.exists()) { return false; }
+
+    // There must be a subfolder called "data"
+    if (!dir.exists("data")) { return false; }
+
+    bool ok = dir.cd("data");
+    Q_ASSERT(ok);
+
+    QStringList nameFiler;
+    nameFiler << "*.png" << "*.vec" << "*.xml";
+    QStringList entries = dir.entryList(nameFiler, QDir::Files);
+
+    return (entries.size() > 0);
+}
+
+Object* FileManager::recoverUnsavedProject(QString intermeidatePath)
+{
+    qDebug() << "TODO: recover project" << intermeidatePath;
+
+    QDir projectDir(intermeidatePath);
+    const QString mainXMLPath = projectDir.filePath(PFF_XML_FILE_NAME);
+    const QString dataFolder = projectDir.filePath(PFF_DATA_DIR);
+
+    std::unique_ptr<Object> object(new Object);
+    object->setWorkingDir(intermeidatePath);
+    object->setMainXMLFile(mainXMLPath);
+    object->setDataDir(dataFolder);
+
+    Status st = recoverObject(object.get());
+    if (!st.ok())
+    {
+        mError = st;
+        return nullptr;
+    }
+    // Transfer ownership to the caller
+    return object.release();
+}
+
+Status FileManager::recoverObject(Object* object)
+{
+    // Check whether the main.xml is fine, if not we should make a valid one.
+    bool mainXmlOK = true;
+
+    QFile file(object->mainXMLFile());
+    mainXmlOK &= file.exists();
+    mainXmlOK &= file.open(QFile::ReadOnly);
+    file.close();
+
+    QDomDocument xmlDoc;
+    mainXmlOK &= xmlDoc.setContent(&file);
+
+    QDomDocumentType type = xmlDoc.doctype();
+    mainXmlOK &= (type.name() == "PencilDocument" || type.name() == "MyObject");
+
+    QDomElement root = xmlDoc.documentElement();
+    mainXmlOK &= (!root.isNull());
+
+    QDomElement objectTag = root.firstChildElement("object");
+    mainXmlOK &= (objectTag.isNull() == false);
+
+    if (mainXmlOK == false)
+    {
+        // the main.xml is broken, try to rebuild one
+        rebuildMainXML(object);
+
+        // Load the newly built main.xml
+        QFile file(object->mainXMLFile());
+        file.open(QFile::ReadOnly);
+        xmlDoc.setContent(&file);
+        root = xmlDoc.documentElement();
+        objectTag = root.firstChildElement("object");
+    }
+    loadPalette(object);
+
+    bool ok = loadObject(object, root);
+    verifyObject(object);
+
+    return ok ? Status::OK : Status::FAIL;
+}
+
+/** Create a new main.xml based on the png/vec filenames left in the data folder */
+Status FileManager::rebuildMainXML(Object* object)
+{
+    QDir dataDir(object->dataDir());
+
+    QStringList nameFiler;
+    nameFiler << "*.png" << "*.vec";
+    const QStringList entries = dataDir.entryList(nameFiler, QDir::Files | QDir::Readable, QDir::Name);
+
+    QMap<int, QStringList> keyFrameGroups;
+
+    // grouping keyframe files by layers
+    for (const QString& s : entries)
+    {
+        int layerIndex = layerIndexFromFilename(s);
+        if (layerIndex > 0)
+        {
+            keyFrameGroups[layerIndex].append(s);
+        }
+    }
+
+    // build the new main XML file
+    const QString mainXMLPath = object->mainXMLFile();
+    QFile file(mainXMLPath);
+    if (!file.open(QFile::WriteOnly | QFile::Text))
+    {
+        return Status::ERROR_FILE_CANNOT_OPEN;
+    }
+
+    QDomDocument xmlDoc("PencilDocument");
+    QDomElement root = xmlDoc.createElement("document");
+    QDomProcessingInstruction encoding = xmlDoc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
+    xmlDoc.appendChild(encoding);
+    xmlDoc.appendChild(root);
+
+    // save editor information
+    QDomElement projDataXml = saveProjectData(object->data(), xmlDoc);
+    root.appendChild(projDataXml);
+
+    // save object
+    QDomElement elemObject = xmlDoc.createElement("object");
+    root.appendChild(elemObject);
+
+    for (const int layerIndex : keyFrameGroups.keys())
+    {
+        const QStringList& frames = keyFrameGroups.value(layerIndex);
+        Status st = rebuildLayerXmlTag(xmlDoc, elemObject, layerIndex, frames);
+    }
+
+    QTextStream fout(&file);
+    xmlDoc.save(fout, 2);
+    fout.flush();
+    file.close();
+
+    return Status::OK;
+}
+/**
+ *  Rebuild a layer xml tag. example:
+ *  @code{.xml}
+ *    <layer id="2" type="2" visibility="1" name="Vector Layer">
+ *      <image src="002.001.vec" frame="1"/>
+ *    </layer>
+ *  @endcode
+ */
+Status FileManager::rebuildLayerXmlTag(QDomDocument& doc,
+                                       QDomElement& elemObject,
+                                       const int layerIndex,
+                                       const QStringList& frames)
+{
+    Q_ASSERT(frames.length() > 0);
+
+    Layer::LAYER_TYPE type = frames[0].endsWith(".png") ? Layer::BITMAP : Layer::VECTOR;
+
+    QDomElement elemLayer = doc.createElement("layer");
+    elemLayer.setAttribute("id", layerIndex + 1); // starts from 1, not 0.
+    elemLayer.setAttribute("name", recoverLayerName(type, layerIndex));
+    elemLayer.setAttribute("visibility", true);
+    elemLayer.setAttribute("type", type);
+    elemObject.appendChild(elemLayer);
+
+    for (const QString& s : frames)
+    {
+        const int framePos = framePosFromFilename(s);
+        if (framePos < 0) { continue; }
+
+        QDomElement elemFrame = doc.createElement("image");
+        elemFrame.setAttribute("frame", framePos);
+        elemFrame.setAttribute("src", s);
+
+        if (type == Layer::BITMAP)
+        {
+            // Since we have no way to know the original img position
+            // Put it at the top left corner of the default camera
+            elemFrame.setAttribute("topLeftX", -800);
+            elemFrame.setAttribute("topLeftY", -600);
+        }
+        elemLayer.appendChild(elemFrame);
+    }
+    return Status::OK;
+}
+
+QString FileManager::recoverLayerName(Layer::LAYER_TYPE type, int index)
+{
+    switch (type)
+    {
+    case Layer::BITMAP:
+        return QString("%1 %2").arg(tr("Bitmap Layer")).arg(index);
+    case Layer::VECTOR:
+        return QString("%1 %2").arg(tr("Vector Layer")).arg(index);
+    case Layer::SOUND:
+        return QString("%1 %2").arg(tr("Sound Layer")).arg(index);
+    default:
+        Q_ASSERT(false);
+    }
+    return "";
+}
+
+int FileManager::layerIndexFromFilename(const QString& filename)
+{
+    const QStringList tokens = filename.split("."); // e.g., 001.019.png or 012.132.vec
+    if (tokens.length() >= 3) // a correct file name must have 3 tokens
+    {
+        return tokens[0].toInt();
+    }
+    return -1;
+}
+
+int FileManager::framePosFromFilename(const QString& filename)
+{
+    const QStringList tokens = filename.split("."); // e.g., 001.019.png or 012.132.vec
+    if (tokens.length() >= 3) // a correct file name must have 3 tokens
+    {
+        return tokens[1].toInt();
+    }
+    return -1;
+}
