@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,7 +25,8 @@ GNU General Public License for more details.
 #include "log.h"
 #include "pencildef.h"
 #include "pencilerror.h"
-#include "colourref.h"
+#include "colorref.h"
+#include "layer.h"
 
 class Object;
 class ObjectData;
@@ -38,14 +39,18 @@ class FileManager : public QObject
 public:
     FileManager(QObject* parent = 0);
 
-    Object* load(QString sFilenNme);
-    Status  save(Object*, QString sFileName);
+    Object* load(const QString& sFilenNme);
+    Status  save(const Object*, const QString& sFileName);
+    Status  writeToWorkingFolder(const Object*);
 
-    QList<ColourRef> loadPaletteFile(QString strFilename);
+    QList<ColorRef> loadPaletteFile(QString strFilename);
     Status error() const { return mError; }
     Status verifyObject(Object* obj);
 
-Q_SIGNALS:
+    QStringList searchForUnsavedProjects();
+    Object* recoverUnsavedProject(QString projectIntermediatePath);
+
+signals:
     void progressChanged(int progress);
     void progressRangeChanged(int maxValue);
 
@@ -56,6 +61,9 @@ private:
     bool loadObjectOldWay(Object*, const QDomElement& root);
     bool isOldForamt(const QString& fileName) const;
     bool loadPalette(Object*);
+    Status writeKeyFrameFiles(const Object* obj, const QString& dataFolder, QStringList& filesWritten);
+    Status writeMainXml(const Object* obj, const QString& mainXml, QStringList& filesWritten);
+    Status writePalette(const Object* obj, const QString& dataFolder, QStringList& filesWritten);
 
     ObjectData* loadProjectData(const QDomElement& element);
     QDomElement saveProjectData(ObjectData*, QDomDocument& xmlDoc);
@@ -68,14 +76,22 @@ private:
 
     void progressForward();
 
+private: // Project recovery
+    bool isProjectRecoverable(const QString& projectFolder);
+    Status recoverObject(Object* object);
+    Status rebuildMainXML(Object* object);
+    Status rebuildLayerXmlTag(QDomDocument& doc, QDomElement& elemObject,
+                              const int layerIndex, const QStringList& frames);
+    QString recoverLayerName(Layer::LAYER_TYPE, int index);
+    int layerIndexFromFilename(const QString& filename);
+    int framePosFromFilename(const QString& filename);
+
 private:
     Status mError = Status::OK;
     QString mstrLastTempFolder;
 
     int mCurrentProgress = 0;
     int mMaxProgressValue = 100;
-
-    QLoggingCategory mLog;
 };
 
 #endif // OBJECTSAVELOADER_H
