@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -57,6 +57,8 @@ public:
     virtual void unloadFile() {}
     virtual bool isLoaded() { return true; }
 
+    virtual quint64 memoryUsage() { return 0; }
+
 private:
     int mFrame = -1;
     int mLength = 1;
