@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -149,7 +149,7 @@ QString LayerBitmap::fileName(KeyFrame* key) const
 }
 
 bool LayerBitmap::needSaveFrame(KeyFrame* key, const QString& savePath)
-{    
+{
     if (key->isModified()) // keyframe was modified
         return true;
     if (QFile::exists(savePath) == false) // hasn't been saved before
@@ -159,9 +159,9 @@ bool LayerBitmap::needSaveFrame(KeyFrame* key, const QString& savePath)
     return false;
 }
 
-QDomElement LayerBitmap::createDomElement(QDomDocument& doc)
+QDomElement LayerBitmap::createDomElement(QDomDocument& doc) const
 {
-    QDomElement layerElem = this->createBaseDomElement(doc);
+    QDomElement layerElem = createBaseDomElement(doc);
 
     foreachKeyFrame([&](KeyFrame* pKeyFrame)
     {
