@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,6 +21,11 @@ GNU General Public License for more details.
 #include <QProgressDialog>
 #include <QApplication>
 #include <QFile>
+#include <QFileInfo>
+#include <QDir>
+#include <QDebug>
+#include <QDateTime>
+#include <QSettings>
 
 #include "layer.h"
 #include "layerbitmap.h"
@@ -39,7 +44,7 @@ GNU General Public License for more details.
 Object::Object(QObject* parent) : QObject(parent)
 {
     setData(new ObjectData());
-    mActiveFramePool.reset(new ActiveFramePool(400));
+    mActiveFramePool.reset(new ActiveFramePool);
 }
 
 Object::~Object()
@@ -63,7 +68,7 @@ void Object::init()
     loadDefaultPalette();
 }
 
-QDomElement Object::saveXML(QDomDocument& doc)
+QDomElement Object::saveXML(QDomDocument& doc) const
 {
     QDomElement objectTag = doc.createElement("object");
 
@@ -82,7 +87,6 @@ bool Object::loadXML(QDomElement docElem, ProgressCallback progressForward)
     {
         return false;
     }
-    int layerNumber = -1;
 
     const QString dataDirPath = mDataDirPath;
 
@@ -91,16 +95,25 @@ bool Object::loadXML(QDomElement docElem, ProgressCallback progressForward)
         QDomElement element = node.toElement(); // try to convert the node to an element.
         if (element.tagName() == "layer")
         {
+            Layer* newLayer;
             switch (element.attribute("type").toInt())
             {
-            case Layer::BITMAP: addNewBitmapLayer(); break;
-            case Layer::VECTOR: addNewVectorLayer(); break;
-            case Layer::SOUND:  addNewSoundLayer();  break;
-            case Layer::CAMERA: addNewCameraLayer(); break;
-            default: Q_ASSERT(false); break;
+            case Layer::BITMAP:
+                newLayer = new LayerBitmap(this);
+                break;
+            case Layer::VECTOR:
+                newLayer = new LayerVector(this);
+                break;
+            case Layer::SOUND:
+                newLayer = new LayerSound(this);
+                break;
+            case Layer::CAMERA:
+                newLayer = new LayerCamera(this);
+                break;
+            default: Q_ASSERT(false); continue;
             }
-            layerNumber++;
-            getLayer(layerNumber)->loadDomElement(element, dataDirPath, progressForward);
+            mLayers.append(newLayer);
+            newLayer->loadDomElement(element, dataDirPath, progressForward);
         }
     }
     return true;
@@ -150,15 +163,15 @@ LayerCamera* Object::addNewCameraLayer()
 
 void Object::createWorkingDir()
 {
-    QString strFolderName;
+    QString projectName;
     if (mFilePath.isEmpty())
     {
-        strFolderName = "Default";
+        projectName = "Default";
     }
     else
     {
         QFileInfo fileInfo(mFilePath);
-        strFolderName = fileInfo.completeBaseName();
+        projectName = fileInfo.completeBaseName();
     }
     QDir dir(QDir::tempPath());
 
@@ -167,7 +180,7 @@ void Object::createWorkingDir()
     {
         strWorkingDir = QString("%1/Pencil2D/%2_%3_%4/")
             .arg(QDir::tempPath())
-            .arg(strFolderName)
+            .arg(projectName)
             .arg(PFF_TMP_DECOMPRESS_EXT)
             .arg(uniqueString(8));
     }
@@ -187,10 +200,18 @@ void Object::deleteWorkingDir() const
     if (!mWorkingDirPath.isEmpty())
     {
         QDir dir(mWorkingDirPath);
-        dir.removeRecursively();
+        bool ok = dir.removeRecursively();
+        Q_ASSERT(ok);
     }
 }
 
+void Object::setWorkingDir(const QString& path)
+{
+    QDir dir(path);
+    Q_ASSERT(dir.exists());
+    mWorkingDirPath = path;
+}
+
 void Object::createDefaultLayers()
 {
     // default layers
@@ -241,6 +262,28 @@ Layer* Object::findLayerByName(QString strName, Layer::LAYER_TYPE type) const
     return nullptr;
 }
 
+Layer* Object::takeLayer(int layerId)
+{
+    // Removes the layer from this Object and returns it
+    // The ownership of this layer has been transfer to the caller
+    int index = -1;
+    for (int i = 0; i< mLayers.length(); ++i)
+    {
+        Layer* layer = mLayers[i];
+        if (layer->id() == layerId)
+        {
+            index = i;
+            break;
+        }
+    }
+
+    if (index == -1) { return nullptr; }
+
+    Layer* layer = mLayers.takeAt(index);
+    layer->setParent(nullptr);
+    return layer;
+}
+
 bool Object::swapLayers(int i, int j)
 {
     if (i < 0 || i >= mLayers.size())
@@ -281,9 +324,24 @@ void Object::deleteLayer(Layer* layer)
     }
 }
 
-ColourRef Object::getColour(int index) const
+bool Object::addLayer(Layer* layer)
 {
-    ColourRef result(Qt::white, "error");
+    if (layer == nullptr)
+    {
+        return false;
+    }
+    if (mLayers.contains(layer))
+    {
+        return false;
+    }
+    layer->setObject(this);
+    mLayers.append(layer);
+    return true;
+}
+
+ColorRef Object::getColor(int index) const
+{
+    ColorRef result(Qt::white, tr("error"));
     if (index > -1 && index < mPalette.size())
     {
         result = mPalette.at(index);
@@ -291,29 +349,46 @@ ColourRef Object::getColour(int index) const
     return result;
 }
 
-void Object::setColour(int index, QColor newColour)
+void Object::setColor(int index, QColor newColor)
 {
     Q_ASSERT(index >= 0);
 
-    mPalette[index].colour = newColour;
+    mPalette[index].color = newColor;
 }
 
-void Object::setColourRef(int index, ColourRef newColourRef)
+void Object::setColorRef(int index, ColorRef newColorRef)
 {
-    mPalette[index] = newColourRef;
+    mPalette[index] = newColorRef;
 }
 
-void Object::addColour(QColor colour)
+void Object::addColor(QColor color)
 {
-    addColour(ColourRef(colour, "Colour " + QString::number(mPalette.size())));
+    addColor(ColorRef(color, tr("Color %1").arg(QString::number(mPalette.size()))));
 }
 
-void Object::addColourAtIndex(int index, ColourRef newColour)
+void Object::movePaletteColor(int start, int end)
 {
-    mPalette.insert(index, newColour);
+    mPalette.move(start, end);
 }
 
-bool Object::isColourInUse(int index)
+void Object::moveVectorColor(int start, int end)
+{
+    for (int i = 0; i < getLayerCount(); i++)
+    {
+        Layer* layer = getLayer(i);
+        if (layer->type() == Layer::VECTOR)
+        {
+            static_cast<LayerVector*>(layer)->moveColor(start, end);
+        }
+    }
+}
+
+void Object::addColorAtIndex(int index, ColorRef newColor)
+{
+    mPalette.insert(index, newColor);
+}
+
+bool Object::isColorInUse(int index)
 {
     for (int i = 0; i < getLayerCount(); i++)
     {
@@ -322,7 +397,7 @@ bool Object::isColourInUse(int index)
         {
             LayerVector* layerVector = static_cast<LayerVector*>(layer);
 
-            if (layerVector->usesColour(index))
+            if (layerVector->usesColor(index))
             {
                 return true;
             }
@@ -332,7 +407,7 @@ bool Object::isColourInUse(int index)
 
 }
 
-void Object::removeColour(int index)
+void Object::removeColor(int index)
 {
     for (int i = 0; i < getLayerCount(); i++)
     {
@@ -340,21 +415,21 @@ void Object::removeColour(int index)
         if (layer->type() == Layer::VECTOR)
         {
             LayerVector* layerVector = static_cast<LayerVector*>(layer);
-            layerVector->removeColour(index);
+            layerVector->removeColor(index);
         }
     }
 
     mPalette.removeAt(index);
 
-    // update the vector pictures using that colour !
+    // update the vector pictures using that color !
 }
 
-void Object::renameColour(int i, QString text)
+void Object::renameColor(int i, QString text)
 {
     mPalette[i].name = text;
 }
 
-QString Object::savePalette(QString dataFolder)
+QString Object::savePalette(const QString& dataFolder) const
 {
     QString fullPath = QDir(dataFolder).filePath("palette.xml");
     bool ok = exportPalette(fullPath);
@@ -363,9 +438,8 @@ QString Object::savePalette(QString dataFolder)
     return "";
 }
 
-void Object::exportPaletteGPL(QFile& file)
+void Object::exportPaletteGPL(QFile& file) const
 {
-
     QString fileName = QFileInfo(file).baseName();
     QTextStream out(&file);
 
@@ -373,15 +447,15 @@ void Object::exportPaletteGPL(QFile& file)
     out << "Name: " << fileName << "\n";
     out << "#" << "\n";
 
-    for (ColourRef ref : mPalette)
+    for (ColorRef ref : mPalette)
     {
-        QColor toRgb = ref.colour.toRgb();
+        QColor toRgb = ref.color.toRgb();
         out << QString("%1 %2 %3").arg(toRgb.red()).arg(toRgb.green()).arg(toRgb.blue());
         out << " " << ref.name << "\n";
     }
 }
 
-void Object::exportPalettePencil(QFile& file)
+void Object::exportPalettePencil(QFile& file) const
 {
     QTextStream out(&file);
 
@@ -390,20 +464,20 @@ void Object::exportPalettePencil(QFile& file)
     doc.appendChild(root);
     for (int i = 0; i < mPalette.size(); i++)
     {
-        ColourRef ref = mPalette.at(i);
-        QDomElement tag = doc.createElement("Colour");
+        ColorRef ref = mPalette.at(i);
+        QDomElement tag = doc.createElement("Color");
         tag.setAttribute("name", ref.name);
-        tag.setAttribute("red", ref.colour.red());
-        tag.setAttribute("green", ref.colour.green());
-        tag.setAttribute("blue", ref.colour.blue());
-        tag.setAttribute("alpha", ref.colour.alpha());
+        tag.setAttribute("red", ref.color.red());
+        tag.setAttribute("green", ref.color.green());
+        tag.setAttribute("blue", ref.color.blue());
+        tag.setAttribute("alpha", ref.color.alpha());
         root.appendChild(tag);
     }
     int IndentSize = 2;
     doc.save(out, IndentSize);
 }
 
-bool Object::exportPalette(QString filePath)
+bool Object::exportPalette(const QString& filePath) const
 {
     QFile file(filePath);
     if (!file.open(QFile::WriteOnly | QFile::Text))
@@ -413,11 +487,9 @@ bool Object::exportPalette(QString filePath)
     }
 
     if (file.fileName().endsWith(".gpl", Qt::CaseInsensitive))
-    {
         exportPaletteGPL(file);
-    } else {
+    else
         exportPalettePencil(file);
-    }
 
     file.close();
     return true;
@@ -429,7 +501,7 @@ bool Object::exportPalette(QString filePath)
  * This should load colors the same as GIMP, with the following intentional exceptions:
  * - Whitespace before and after a name does not appear in the name
  * - The last line is processed, even if there is not a trailing newline
- * - Colours without a name will use our automatic naming system rather than "Untitled"
+ * - Colors without a name will use our automatic naming system rather than "Untitled"
  */
 void Object::importPaletteGPL(QFile& file)
 {
@@ -455,9 +527,9 @@ void Object::importPaletteGPL(QFile& file)
         }
     }
 
-    // Colours inherit the value from the previous colour for missing channels
+    // Colors inherit the value from the previous color for missing channels
     // Some palettes may rely on this behavior so we should try to replicate it
-    QColor prevColour(Qt::black);
+    QColor prevColor(Qt::black);
 
     do
     {
@@ -493,20 +565,20 @@ void Object::importPaletteGPL(QFile& file)
         // trim additional spaces
         name = name.trimmed();
 
-        // Get values from previous colour if necessary
-        if (countInLine < 2) green = prevColour.green();
-        if (countInLine < 3) blue = prevColour.blue();
+        // Get values from previous color if necessary
+        if (countInLine < 2) green = prevColor.green();
+        if (countInLine < 3) blue = prevColor.blue();
 
-        // GIMP assigns colours the name "Untitled" by default now
+        // GIMP assigns colors the name "Untitled" by default now
         // so in addition to missing names, we also use automatic
         // naming for this
         if (name.isEmpty() || name == "Untitled") name = QString();
 
-        QColor colour(red, green, blue);
-        if (colour.isValid())
+        QColor color(red, green, blue);
+        if (color.isValid())
         {
-            mPalette.append(ColourRef(colour, name));
-            prevColour = colour;
+            mPalette.append(ColorRef(color, name));
+            prevColor = color;
         }
     } while (in.readLineInto(&line));
 }
@@ -516,7 +588,6 @@ void Object::importPalettePencil(QFile& file)
     QDomDocument doc;
     doc.setContent(&file);
 
-    mPalette.clear();
     QDomElement docElem = doc.documentElement();
     QDomNode tag = docElem.firstChild();
     while (!tag.isNull())
@@ -529,12 +600,26 @@ void Object::importPalettePencil(QFile& file)
             int g = e.attribute("green").toInt();
             int b = e.attribute("blue").toInt();
             int a = e.attribute("alpha", "255").toInt();
-            mPalette.append(ColourRef(QColor(r, g, b, a), name));
+            mPalette.append(ColorRef(QColor(r, g, b, a), name));
         }
         tag = tag.nextSibling();
     }
 }
 
+void Object::openPalette(QString filePath)
+{
+    if (!QFile::exists(filePath))
+    {
+        return;
+    }
+
+    mPalette.clear();
+    importPalette(filePath);
+}
+
+/*
+ * Imports palette, e.g. appends to palette
+*/
 bool Object::importPalette(QString filePath)
 {
     QFile file(filePath);
@@ -558,30 +643,30 @@ bool Object::importPalette(QString filePath)
 void Object::loadDefaultPalette()
 {
     mPalette.clear();
-    addColour(ColourRef(QColor(Qt::black), QString(tr("Black"))));
-    addColour(ColourRef(QColor(Qt::red), QString(tr("Red"))));
-    addColour(ColourRef(QColor(Qt::darkRed), QString(tr("Dark Red"))));
-    addColour(ColourRef(QColor(255, 128, 0), QString(tr("Orange"))));
-    addColour(ColourRef(QColor(128, 64, 0), QString(tr("Dark Orange"))));
-    addColour(ColourRef(QColor(Qt::yellow), QString(tr("Yellow"))));
-    addColour(ColourRef(QColor(Qt::darkYellow), QString(tr("Dark Yellow"))));
-    addColour(ColourRef(QColor(Qt::green), QString(tr("Green"))));
-    addColour(ColourRef(QColor(Qt::darkGreen), QString(tr("Dark Green"))));
-    addColour(ColourRef(QColor(Qt::cyan), QString(tr("Cyan"))));
-    addColour(ColourRef(QColor(Qt::darkCyan), QString(tr("Dark Cyan"))));
-    addColour(ColourRef(QColor(Qt::blue), QString(tr("Blue"))));
-    addColour(ColourRef(QColor(Qt::darkBlue), QString(tr("Dark Blue"))));
-    addColour(ColourRef(QColor(255, 255, 255), QString(tr("White"))));
-    addColour(ColourRef(QColor(220, 220, 229), QString(tr("Very Light Grey"))));
-    addColour(ColourRef(QColor(Qt::lightGray), QString(tr("Light Grey"))));
-    addColour(ColourRef(QColor(Qt::gray), QString(tr("Grey"))));
-    addColour(ColourRef(QColor(Qt::darkGray), QString(tr("Dark Grey"))));
-    addColour(ColourRef(QColor(255, 227, 187), QString(tr("Light Skin"))));
-    addColour(ColourRef(QColor(221, 196, 161), QString(tr("Light Skin - shade"))));
-    addColour(ColourRef(QColor(255, 214, 156), QString(tr("Skin"))));
-    addColour(ColourRef(QColor(207, 174, 127), QString(tr("Skin - shade"))));
-    addColour(ColourRef(QColor(255, 198, 116), QString(tr("Dark Skin"))));
-    addColour(ColourRef(QColor(227, 177, 105), QString(tr("Dark Skin - shade")) ));
+    addColor(ColorRef(QColor(Qt::black), QString(tr("Black"))));
+    addColor(ColorRef(QColor(Qt::red), QString(tr("Red"))));
+    addColor(ColorRef(QColor(Qt::darkRed), QString(tr("Dark Red"))));
+    addColor(ColorRef(QColor(255, 128, 0), QString(tr("Orange"))));
+    addColor(ColorRef(QColor(128, 64, 0), QString(tr("Dark Orange"))));
+    addColor(ColorRef(QColor(Qt::yellow), QString(tr("Yellow"))));
+    addColor(ColorRef(QColor(Qt::darkYellow), QString(tr("Dark Yellow"))));
+    addColor(ColorRef(QColor(Qt::green), QString(tr("Green"))));
+    addColor(ColorRef(QColor(Qt::darkGreen), QString(tr("Dark Green"))));
+    addColor(ColorRef(QColor(Qt::cyan), QString(tr("Cyan"))));
+    addColor(ColorRef(QColor(Qt::darkCyan), QString(tr("Dark Cyan"))));
+    addColor(ColorRef(QColor(Qt::blue), QString(tr("Blue"))));
+    addColor(ColorRef(QColor(Qt::darkBlue), QString(tr("Dark Blue"))));
+    addColor(ColorRef(QColor(255, 255, 255), QString(tr("White"))));
+    addColor(ColorRef(QColor(220, 220, 229), QString(tr("Very Light Grey"))));
+    addColor(ColorRef(QColor(Qt::lightGray), QString(tr("Light Grey"))));
+    addColor(ColorRef(QColor(Qt::gray), QString(tr("Grey"))));
+    addColor(ColorRef(QColor(Qt::darkGray), QString(tr("Dark Grey"))));
+    addColor(ColorRef(QColor(255, 227, 187), QString(tr("Pale Orange Yellow"))));
+    addColor(ColorRef(QColor(221, 196, 161), QString(tr("Pale Grayish Orange Yellow"))));
+    addColor(ColorRef(QColor(255, 214, 156), QString(tr("Orange Yellow "))));
+    addColor(ColorRef(QColor(207, 174, 127), QString(tr("Grayish Orange Yellow"))));
+    addColor(ColorRef(QColor(255, 198, 116), QString(tr("Light Orange Yellow"))));
+    addColor(ColorRef(QColor(227, 177, 105), QString(tr("Light Grayish Orange Yellow")) ));
 }
 
 void Object::paintImage(QPainter& painter,int frameNumber,
@@ -617,8 +702,10 @@ void Object::paintImage(QPainter& painter,int frameNumber,
                 LayerBitmap* layerBitmap = static_cast<LayerBitmap*>(layer);
 
                 BitmapImage* bitmap = layerBitmap->getLastBitmapImageAtFrame(frameNumber);
-                if (bitmap)
+                if (bitmap != nullptr)
+                {
                     bitmap->paintImage(painter);
+                }
 
             }
             // paints the vector images
@@ -626,8 +713,10 @@ void Object::paintImage(QPainter& painter,int frameNumber,
             {
                 LayerVector* layerVector = static_cast<LayerVector*>(layer);
                 VectorImage* vec = layerVector->getLastVectorImageAtFrame(frameNumber, 0);
-                if (vec)
+                if (vec != nullptr)
+                {
                     vec->paintImage(painter, false, false, antialiasing);
+                }
             }
         }
     }
@@ -664,16 +753,16 @@ QString Object::copyFileToDataFolder(QString strFilePath)
 }
 
 bool Object::exportFrames(int frameStart, int frameEnd,
-                          LayerCamera* cameraLayer,
+                          const LayerCamera* cameraLayer,
                           QSize exportSize,
                           QString filePath,
                           QString format,
                           bool transparency,
                           bool exportKeyframesOnly,
-                          QString layerName,
+                          const QString& layerName,
                           bool antialiasing,
                           QProgressDialog* progress = nullptr,
-                          int progressMax = 50)
+                          int progressMax = 50) const
 {
     Q_ASSERT(cameraLayer);
 
@@ -774,7 +863,7 @@ bool Object::exportX(int frameStart, int frameEnd, QTransform view, QSize export
             xPainter.setWorldTransform(thumbView);
             xPainter.setClipRegion(thumbView.inverted().map(QRegion(target)));
             paintImage(xPainter, i, false, antialiasing);
-            xPainter.resetMatrix();
+            xPainter.resetTransform();
             xPainter.setClipping(false);
             xPainter.setPen(Qt::black);
             xPainter.drawRect(target);
@@ -796,7 +885,7 @@ bool Object::exportX(int frameStart, int frameEnd, QTransform view, QSize export
     return true;
 }
 
-bool Object::exportIm(int frame, QTransform view, QSize cameraSize, QSize exportSize, QString filePath, QString format, bool antialiasing, bool transparency)
+bool Object::exportIm(int frame, QTransform view, QSize cameraSize, QSize exportSize, QString filePath, QString format, bool antialiasing, bool transparency) const
 {
     QImage imageToExport(exportSize, QImage::Format_ARGB32_Premultiplied);
 
@@ -822,7 +911,7 @@ int Object::getLayerCount() const
     return mLayers.size();
 }
 
-ObjectData* Object::data()
+ObjectData* Object::data() const
 {
     Q_ASSERT(mData != nullptr);
     return mData.get();
@@ -834,10 +923,10 @@ void Object::setData(ObjectData* d)
     mData.reset(d);
 }
 
-int Object::totalKeyFrameCount()
+int Object::totalKeyFrameCount() const
 {
     int sum = 0;
-    for (Layer* layer : mLayers)
+    for (const Layer* layer : mLayers)
     {
         sum += layer->keyFrameCount();
     }
@@ -846,8 +935,12 @@ int Object::totalKeyFrameCount()
 
 void Object::updateActiveFrames(int frame) const
 {
-    int beginFrame = std::max(frame - 3, 1);
-    int endFrame = frame + 4;
+    const int beginFrame = std::max(frame - 3, 1);
+    const int endFrame = frame + 4;
+
+    const int minFrameCount = getLayerCount() * (endFrame - beginFrame);
+    mActiveFramePool->setMinFrameCount(minFrameCount);
+
     for (int i = 0; i < getLayerCount(); ++i)
     {
         Layer* layer = getLayer(i);
@@ -859,7 +952,8 @@ void Object::updateActiveFrames(int frame) const
     }
 }
 
-void Object::setActiveFramePoolSize(int n)
+void Object::setActiveFramePoolSize(int sizeInMB)
 {
-    mActiveFramePool->resize(n);
+    // convert MB to Byte
+    mActiveFramePool->resize(qint64(sizeInMB) * 1024 * 1024);
 }
