@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -16,10 +16,11 @@ GNU General Public License for more details.
 */
 
 #include "objectdata.h"
+#include <QSettings>
 
 ObjectData::ObjectData()
 {
-    QSettings settings(PENCIL2D,PENCIL2D);
+    QSettings settings(PENCIL2D, PENCIL2D);
     mFps = settings.value(SETTING_FPS).toInt();
     if (mFps < 1)
         mFps = 12;
