@@ -1,8 +1,8 @@
 /*
 
-Pencil - Traditional Animation Software
+Pencil2D - Traditional Animation Software
 Copyright (C) 2005-2007 Patrick Corrieri & Pascal Naidon
-Copyright (C) 2012-2018 Matthew Chiawen Chang
+Copyright (C) 2012-2020 Matthew Chiawen Chang
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -232,7 +232,7 @@ QCursor BaseTool::selectMoveCursor(MoveMode mode, ToolType type)
                 break;
             }
             default:
-                return (type == SELECT) ? Qt::CrossCursor : Qt::ArrowCursor;
+                return (type == SELECT) ? QCursor(QPixmap(":icons/cross.png"), 10, 10) : Qt::ArrowCursor;
                 break;
         }
         cursorPainter.end();
