@@ -26,6 +26,7 @@
 #include <QLineF>
 #include <QPainterPath>
 #include "object.h"
+#include "pointerevent.h"
 
 
 StrokeManager::StrokeManager()
@@ -34,7 +35,6 @@ StrokeManager::StrokeManager()
 
     mTabletInUse = false;
     mTabletPressure = 0;
-    mMeanPressure = 0;
 
     reset();
     connect(&timer, &QTimer::timeout, this, &StrokeManager::interpolatePollAndPaint);
@@ -46,7 +46,7 @@ void StrokeManager::reset()
     pressureQueue.clear();
     strokeQueue.clear();
     pressure = 0.0f;
-    hasTangent = false;
+    mHasTangent = false;
     timer.stop();
     mStabilizerLevel = -1;
 }
@@ -56,114 +56,88 @@ void StrokeManager::setPressure(float pressure)
     mTabletPressure = pressure;
 }
 
-QPointF StrokeManager::getEventPosition(QMouseEvent* event)
+void StrokeManager::pointerPressEvent(PointerEvent* event)
 {
-    QPointF pos;
-
-    if ( mTabletInUse )
+    reset();
+    if (!(event->button() == Qt::NoButton)) // if the user is pressing the left/right button
     {
-        // QT BUG (Wacom Tablets): updates are not synchronised in Windows giving different coordinates.
-        // Clue: Not a Microsoft nor Wacom problem because other windows apps are working fine in the same tablet mode.
-        // Solved: Qt bug in Wacom coding -> a lot of patches but no real solutions.
-        // QPointF pos2 = event->pos() + mTabletPosition - event->globalPos();
-        // Patch: next line skips the coordinate problem and it seems safe .
-        pos = event->pos() + mTabletPosition - mTabletPosition.toPoint();
-        //pos = event->pos();
-        //qDebug() << "New pos" << pos << ", Old pos" << pos2;
+        //qDebug() << "press";
+        mLastPressPixel = mCurrentPressPixel;
+        mCurrentPressPixel = event->posF();
+    }
+
+    mLastPixel = mCurrentPixel = event->posF();
+
+    mStrokeStarted = true;
+    setPressure(event->pressure());
+}
+
+void StrokeManager::pointerMoveEvent(PointerEvent* event)
+{
+    // only applied to drawing tools.
+    if (mStabilizerLevel != -1)
+    {
+        smoothMousePos(event->posF());
     }
     else
     {
-        pos = event->localPos();
+        // No smoothing
+        mLastPixel = mCurrentPixel;
+        mCurrentPixel = event->posF();
+        mLastInterpolated = mCurrentPixel;
     }
-
-    return pos;
-}
-
-void StrokeManager::mousePressEvent(QMouseEvent* event)
-{
-    reset();
-    if ( !(event->button() == Qt::NoButton) ) // if the user is pressing the left/right button
+    if(event->isTabletEvent())
     {
-        mLastPressPixel = getEventPosition(event);
+        setPressure(event->pressure());
     }
-    mLastPixel = getEventPosition( event );
-    mCurrentPixel = getEventPosition( event );
-
-    mStrokeStarted = true;
-
 }
 
-void StrokeManager::mouseReleaseEvent(QMouseEvent* event)
+void StrokeManager::pointerReleaseEvent(PointerEvent* event)
 {
     // flush out stroke
-    if ( mStrokeStarted )
+    if (mStrokeStarted)
     {
-        mouseMoveEvent(event);
+        pointerMoveEvent(event);
     }
 
     mStrokeStarted = false;
 }
 
-void StrokeManager::tabletEvent(QTabletEvent* event)
-{
-    if (event->type() == QEvent::TabletPress) { mTabletInUse = true; }
-    if (event->type() == QEvent::TabletRelease) { mTabletInUse = false; }
-
-    mTabletPosition = event->posF();
-    setPressure(event->pressure());
-}
-
 void StrokeManager::setStabilizerLevel(int level)
 {
     mStabilizerLevel = level;
 }
 
-void StrokeManager::mouseMoveEvent(QMouseEvent* event)
-{
-    QPointF pos = getEventPosition(event);
-
-    // only applied to drawing tools.
-    if (mStabilizerLevel != -1){
-        smoothMousePos(pos);
-    } else {
-        // No smoothing
-        mLastPixel = mCurrentPixel;
-        mCurrentPixel = pos;
-        mLastInterpolated = mCurrentPixel;
-
-    }
-}
-
 void StrokeManager::smoothMousePos(QPointF pos)
 {
-
     // Smooth mouse position before drawing
     QPointF smoothPos;
 
-    if (mStabilizerLevel == StabilizationLevel::NONE) {
-
+    if (mStabilizerLevel == StabilizationLevel::NONE)
+    {
         mLastPixel = mCurrentPixel;
         mCurrentPixel = pos;
         mLastInterpolated = mCurrentPixel;
     }
-    else if (mStabilizerLevel == StabilizationLevel::SIMPLE) {
-
+    else if (mStabilizerLevel == StabilizationLevel::SIMPLE)
+    {
         // simple interpolation
-        smoothPos = QPointF( ( pos.x() + mCurrentPixel.x() ) / 2.0, ( pos.y() + mCurrentPixel.y() ) / 2.0 );
+        smoothPos = QPointF((pos.x() + mCurrentPixel.x()) / 2.0, (pos.y() + mCurrentPixel.y()) / 2.0);
         mLastPixel = mCurrentPixel;
         mCurrentPixel = smoothPos;
         mLastInterpolated = mCurrentPixel;
 
         // shift queue
-        while ( strokeQueue.size()  >= STROKE_QUEUE_LENGTH )
+        while (strokeQueue.size() >= STROKE_QUEUE_LENGTH)
         {
             strokeQueue.pop_front();
         }
 
-        strokeQueue.push_back( smoothPos );
-    } else if (mStabilizerLevel == StabilizationLevel::STRONG ) {
-
-        smoothPos = QPointF( ( pos.x() + mLastInterpolated.x() ) / 2.0, ( pos.y() + mLastInterpolated.y() ) / 2.0 );
+        strokeQueue.push_back(smoothPos);
+    }
+    else if (mStabilizerLevel == StabilizationLevel::STRONG)
+    {
+        smoothPos = QPointF((pos.x() + mLastInterpolated.x()) / 2.0, (pos.y() + mLastInterpolated.y()) / 2.0);
 
         mLastInterpolated = mCurrentPixel;
         mCurrentPixel = smoothPos;
@@ -172,7 +146,7 @@ void StrokeManager::smoothMousePos(QPointF pos)
 
     mousePos = pos;
 
-    if ( !mStrokeStarted )
+    if (!mStrokeStarted)
     {
         return;
     }
@@ -186,47 +160,50 @@ void StrokeManager::smoothMousePos(QPointF pos)
 
 QPointF StrokeManager::interpolateStart(QPointF firstPoint)
 {
-        if (mStabilizerLevel == StabilizationLevel::SIMPLE) {
-            // Clear queue
-            strokeQueue.clear();
-            pressureQueue.clear();
+    if (mStabilizerLevel == StabilizationLevel::SIMPLE)
+    {
+        // Clear queue
+        strokeQueue.clear();
+        pressureQueue.clear();
 
-            mSingleshotTime.start();
-            previousTime = mSingleshotTime.elapsed();
+        mSingleshotTime.start();
+        previousTime = mSingleshotTime.elapsed();
 
-            mLastPixel = firstPoint;
+        mLastPixel = firstPoint;
+    }
+    else if (mStabilizerLevel == StabilizationLevel::STRONG)
+    {
+        mSingleshotTime.start();
+        previousTime = mSingleshotTime.elapsed();
+
+        // Clear queue
+        strokeQueue.clear();
+        pressureQueue.clear();
+    
+        const int sampleSize = 5;
+        Q_ASSERT(sampleSize > 0);
+
+        // fill strokeQueue with firstPoint x times
+        for (int i = sampleSize; i > 0; i--)
+        {
+            strokeQueue.enqueue(firstPoint);
         }
-        else if (mStabilizerLevel == StabilizationLevel::STRONG){
 
-            mSingleshotTime.start();
-            previousTime = mSingleshotTime.elapsed();
+        // last interpolated stroke should always be firstPoint
+        mLastInterpolated = firstPoint;
 
-            int sampleSize = 5;
+        // draw and poll each millisecond
+        timer.setInterval(sampleSize);
+        timer.start();
+    }
+    else if (mStabilizerLevel == StabilizationLevel::NONE)
+    {
+        // Clear queue
+        strokeQueue.clear();
+        pressureQueue.clear();
 
-            // Clear queue
-            strokeQueue.clear();
-            pressureQueue.clear();
-
-            assert(sampleSize > 0);
-
-            // fill strokeQueue with firstPoint x times
-            for ( int i = sampleSize; i > 0; i--) {
-                strokeQueue.enqueue(firstPoint);
-            }
-
-            // last interpolated stroke should always be firstPoint
-            mLastInterpolated = firstPoint;
-
-            // draw and poll each millisecond
-            timer.setInterval(sampleSize);
-            timer.start();
-        } else if (mStabilizerLevel == StabilizationLevel::NONE) {
-            // Clear queue
-            strokeQueue.clear();
-            pressureQueue.clear();
-
-            mLastPixel = firstPoint;
-        }
+        mLastPixel = firstPoint;
+    }
     return firstPoint;
 }
 
@@ -254,23 +231,22 @@ QList<QPointF> StrokeManager::interpolateStroke()
     // is nan initially
     QList<QPointF> result;
 
-    qreal x = 0,
-          y = 0,
-          pressure = 0;
-
-    if (mStabilizerLevel == StabilizationLevel::SIMPLE) {
-
+    if (mStabilizerLevel == StabilizationLevel::SIMPLE)
+    {
         result = tangentInpolOp(result);
 
     }
-    else if (mStabilizerLevel == StabilizationLevel::STRONG){
-
+    else if (mStabilizerLevel == StabilizationLevel::STRONG)
+    {
+        qreal x = 0;
+        qreal y = 0;
+        qreal pressure = 0;
         result = meanInpolOp(result, x, y, pressure);
 
-    } else if (mStabilizerLevel == StabilizationLevel::NONE) {
-
+    }
+    else if (mStabilizerLevel == StabilizationLevel::NONE)
+    {
         result = noInpolOp(result);
-
     }
     return result;
 }
@@ -292,13 +268,13 @@ QList<QPointF> StrokeManager::tangentInpolOp(QList<QPointF> points)
 {
     int time = mSingleshotTime.elapsed();
     static const qreal smoothness = 1.f;
-    QLineF line( mLastPixel, mCurrentPixel);
+    QLineF line(mLastPixel, mCurrentPixel);
 
     qreal scaleFactor = line.length() * 3.f;
 
-    if ( !hasTangent && scaleFactor > 0.01f)
+    if (!mHasTangent && scaleFactor > 0.01f)
     {
-        hasTangent = true;
+        mHasTangent = true;
         /*
         qDebug() << "scaleFactor" << scaleFactor
                  << "current pixel " << mCurrentPixel
@@ -306,11 +282,11 @@ QList<QPointF> StrokeManager::tangentInpolOp(QList<QPointF> points)
          */
         m_previousTangent = (mCurrentPixel - mLastPixel) * smoothness / (3.0 * scaleFactor);
         //qDebug() << "previous tangent" << m_previousTangent;
-        QLineF _line(QPointF(0,0), m_previousTangent);
+        QLineF _line(QPointF(0, 0), m_previousTangent);
         // don't bother for small tangents, as they can induce single pixel wobbliness
         if (_line.length() < 2)
         {
-            m_previousTangent = QPointF(0,0);
+            m_previousTangent = QPointF(0, 0);
         }
     }
     else
@@ -320,7 +296,7 @@ QList<QPointF> StrokeManager::tangentInpolOp(QList<QPointF> points)
         //qDebug() << "scalefactor1=" << scaleFactor << m_previousTangent << newTangent;
         if (scaleFactor == 0)
         {
-            newTangent = QPointF(0,0);
+            newTangent = QPointF(0, 0);
         }
         else
         {
@@ -334,37 +310,32 @@ QList<QPointF> StrokeManager::tangentInpolOp(QList<QPointF> points)
         //c1 = mLastPixel;
         //c2 = mCurrentPixel;
         points << mLastPixel << c1 << c2 << mCurrentPixel;
-        /*
-        qDebug() << mLastPixel
-                 << c1
-                 << c2
-                 << mCurrentPixel;
-         */
+        //qDebug() << mLastPixel << c1 << c2 << mCurrentPixel;
         m_previousTangent = newTangent;
     }
 
     previousTime = time;
     return points;
-
 }
 
 // Mean sampling interpolation operation
 QList<QPointF> StrokeManager::meanInpolOp(QList<QPointF> points, qreal x, qreal y, qreal pressure)
 {
-    for (int i = 0; i < strokeQueue.size(); i++) {
-           x += strokeQueue[i].x();
-           y += strokeQueue[i].y();
-           pressure += getPressure();
+    for (int i = 0; i < strokeQueue.size(); i++)
+    {
+        x += strokeQueue[i].x();
+        y += strokeQueue[i].y();
+        pressure += getPressure();
     }
 
-    // get arichmic mean of x, y and pressure
+    // get arithmetic mean of x, y and pressure
     x /= strokeQueue.size();
     y /= strokeQueue.size();
     pressure /= strokeQueue.size();
 
     // Use our interpolated points
     QPointF mNewInterpolated = mLastInterpolated;
-    mNewInterpolated = QPointF(x,y);
+    mNewInterpolated = QPointF(x, y);
 
     points << mLastPixel << mLastInterpolated << mNewInterpolated << mCurrentPixel;
 
@@ -379,22 +350,20 @@ void StrokeManager::interpolateEnd()
 {
     // Stop timer
     timer.stop();
-    if (mStabilizerLevel == StabilizationLevel::STRONG) {
+    if (mStabilizerLevel == StabilizationLevel::STRONG)
+    {
         if (!strokeQueue.isEmpty())
         {
-
             // How many samples should we get point from?
             // TODO: Qt slider.
             int sampleSize = 5;
 
-            assert(sampleSize > 0);
+            Q_ASSERT(sampleSize > 0);
             for (int i = sampleSize; i > 0; i--)
             {
                 interpolatePoll();
                 interpolateStroke();
             }
-        } else {
-            // Do nothing
         }
     }
 }
