@@ -2,35 +2,34 @@
 
 set -e
 
-git remote set-branches --add origin master
-git fetch
+if [[ $branch == "release" ]]; then
+  exit 0
+fi
 
 commit=$TRAVIS_COMMIT
 branch=$TRAVIS_BRANCH
-CHANGED_FILES=`git diff --name-only origin/master...${commit}`
-
-if [[ $branch == "master" ]]; then
-  CHANGED_FILES=`git diff --name-only HEAD^ HEAD`
-fi
+commit_range=$TRAVIS_COMMIT_RANGE
 
 echo "branch is: $branch"
 
+printf "git diff --name-only %s\n" "${commit_range}"
+CHANGED_FILES=`git diff --name-only ${commit_range}`
+
+printf "%s\n" "${CHANGED_FILES}"
+printf "=============\n"
+
 ONLY_READMES=True
 MD=".md"
 SH=".sh"
 PY=".py"
 PS1=".ps1"
 
-printf "git diff --name-only origin/master...%s\n" "${TRAVIS_COMMIT}"
-printf "%s\n" "${CHANGED_FILES}"
-printf "=============\n"
-
 for CHANGED_FILE in $CHANGED_FILES; do
-  printf "Check ${CHANGED_FILES}"
+  #printf "Check ${CHANGED_FILES}\n"
   if ! [[ $CHANGED_FILE =~ $MD ||
-  		  $CHANGED_FILE =~ $SH ||
-  		  $CHANGED_FILE =~ $PY ||
-  		  $CHANGED_FILE =~ $PS1 ]] ; then
+          $CHANGED_FILE =~ $SH ||
+          $CHANGED_FILE =~ $PY ||
+          $CHANGED_FILE =~ $PS1 ]] ; then
     ONLY_READMES=False
     printf "Gotcha! %s\n" "${CHANGED_FILE}"
     break
